/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */

#ifndef DEBUG_H
#define DEBUG_H

#include <sys/time.h>
#ifdef __cplusplus
extern "C" {
#endif

extern int debug_on;

typedef struct iodstat iodstat, *iodstat_p;

struct iodstat {
	struct timeval st_wait;
	struct timeval st_serv;
	struct timeval end_serv;
	int	 type;
	int	 numaccess;
	int	 fd;
	int 	 offset; 
};

/* 
   on LOG() and ERR(), I didn't like the LOG1() ... LOG4(), and 
   made LOG() 'n' ERR() into varargs macros. 

   In case of not gcc, I also made a func replacement, but it is less
   efficient.

   I'm lazy enought to think that \n at the end of LOG() and ERR() is
   implied, so I made a func to to check if a string has a \n at the
   end.

   The reason is that a \n at the end of a log message is implied.
   A multi line log message shoud be done my a single call to LOG(). 
   if you don't you're not thread safe, nor can multiple progs share a log file.

   Terje 
*/

#ifdef __GNUC__
#define ___INLINE inline
#else 
#define ___INLINE
#endif

static ___INLINE int _cknl(char * sz, ...)
{
  int i = 0;
  if (sz == NULL) return 0;
  while (sz[i] != '\0') i++;
  i--;
  if (sz[i] == '\n') return 1;
  return 0;
};

#undef ___INLINE

#ifdef DEBUG

#ifdef __GNUC__
#define LOG(x...) do { if (debug_on>0) { fprintf(stderr, x); \
if (!_cknl(x)) fputc('\n', stderr); fflush(stderr); } } while(0)
#else 
#include <stdarg.h>
static void LOG(char * fmt, ...) 
{
  va_list ap;
  va_start(ap, fmt);

  if (debug_on <= 0) return;

  vfprintf(stderr, fmt, ap);
  if (!_cknl(fmt)) fputc('\n', stderr); 
  fflush(stderr);
  return;
};
#endif /* __GNUC__ */

#define LOG1(x,y) if (debug_on>0) { fprintf(stderr, x, (y)); fflush(stderr); }
#define LOG2(x,y,z) if (debug_on>0) { fprintf(stderr, x, (y), (z)); \
	fflush(stderr); }
#define LOG3(w,x,y,z) if (debug_on>0) { fprintf(stderr, w, (x), (y), (z)); \
	fflush(stderr); }
#define LOG4(v,w,x,y,z) if (debug_on>0) { fprintf(stderr,v,(w),(x),(y),(z)); \
	fflush(stderr); }
#else /* DEBUG */

#ifdef __GNUC__
#define LOG(x...);
#else
#include <stdarg.h>
static void LOG(char * fmt, ...) { ; };
#endif
#define LOG1(x,y);
#define LOG2(x,y,z);
#define LOG3(w,x,y,z);
#define LOG4(v,w,x,y,z);
#endif /* DEBUG */

#if 1

#ifdef __GNUC__

#define ERR(x...) do { fprintf(stderr, x); \
if (!_cknl(x)) fputc('\n', stderr); fflush(stderr); }  while(0)
#else 
#include <stdarg.h>
static void ERR(char * fmt, ...) 
{
  va_list ap;
  va_start(ap, fmt);

  vfprintf(stderr, fmt, ap);
  if (!_cknl(fmt)) fputc('\n', stderr); 
  fflush(stderr);
  return;
};
#endif /* __GNUC__ */

#define ERR1(x,y) { fprintf(stderr, x, (y)); fflush(stderr); }
#define ERR2(x,y,z) { fprintf(stderr, x, (y), (z)); \
	fflush(stderr); }
#define ERR3(w,x,y,z) { fprintf(stderr, w, (x), (y), (z)); \
	fflush(stderr); }
#else
#ifdef __GNUC__
#else
#include <stdarg.h>
static void ERR(char * fmt, ...) { ; };
#endif
#define ERR(x...);
#define ERR1(x,y);
#define ERR2(x,y,z);
#define ERR3(w,x,y,z);
#endif

#if 1
#define PERROR(x) { perror(x); fflush(stderr); }
#else
#define PERROR(x);
#endif

#ifdef __cplusplus
}
#endif

#endif
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
