/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/*
 * FLIST.H - header file for iod file management functions
 *
 */

#ifndef FLIST_H
#define FLIST_H

#include <config.h>
#include <llist.h>
#include <meta.h>
#include <sys/stat.h>
#include <unistd.h>

typedef llist_p flist_p;

typedef struct finfo finfo, *finfo_p;
typedef struct ginfo ginfo, *ginfo_p;
typedef struct minfo minfo, *minfo_p;

#ifdef HAVE_MADVISE
#define MADVISE_START 4
#endif

/* mmapped file info */
struct minfo {
	void *mloc;			/* address of mmapped region */
	int64_t msize;			/* size of region */
	int64_t moff;			/* offset into file */
	int mprot;			/* protection of mmapped region */
#ifdef HAVE_MADVISE
        int madv;			/* status of madvise */
#endif
};

/* group connection info */
struct ginfo {
	int sched;			/* socket # of scheduler connection */
	int nr_conn;		/* # of data connections (0 for normal access) */
	int *conn;			/* array of sockets corresponding to data conn's */
};

struct finfo {
	int64_t f_ino;      /* inode # of metadata file */
	int32_t cap;          /* capability */
	int fd;				/* file desc. # */
	int sock;			/* socket # for non-group opens, -1 for group opens */
	pvfs_filestat p_stat; /* PVFS metadata for file */
	int64_t fsize;			/* file size */
	int pnum;			/* partition # */
	minfo mm;			/* mmap specs */
	ginfo grp;			/* group specs */
};

flist_p flist_new(void);
int f_add(flist_p, finfo_p);
int f_sock_rem(flist_p fl_p, int s);
finfo_p f_ino_cap_search(flist_p, int64_t f_ino, int32_t cap);
int f_ino_cap_rem(flist_p, int64_t f_ino, int32_t cap);
void flist_cleanup(flist_p);
int f_dump(void *);
finfo_p f_new(void);
void f_free(void *);
int flist_purge_sock(flist_p fl_p, int s);
int flist_dump(flist_p fl_p);

#define f_search(_x,_y,_z) f_ino_cap_search((_x), (_y), (_z))
#define f_rem(_x,_y,_z) f_ino_cap_rem((_x), (_y), (_z))

#endif
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
