/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


#ifndef JLIST_H
#define JLIST_H

#include <config.h>
#include <llist.h>
#include <alist.h>

/* includes needed for using fd_sets */
#include <dfd_set.h>

typedef llist_p jlist_p;

typedef struct jinfo jinfo, *jinfo_p;

struct jinfo {
	int jobnum;			/* unique identifier for jobs			*/
	int8_t type;
	int8_t nospc;		/* no space left in file -- throw away any write data */
	dyn_fdset socks;		/* list of sockets that are (or have been) in job */
	int max_nr;			/* max socket # + 1 (for use w/socks) */
	int64_t size;			/* amount of data to be moved by this job */
	finfo_p file_p;	/* pointer to open file info */
	int64_t lastb;			/* offset of last byte in file to be accessed by job */
	alist_p al_p;		/* list of accesses */
};

jlist_p jlist_new(void);
int j_add(jlist_p, jinfo_p);
jinfo_p j_search(jlist_p, int sock);
int j_sock_rem(jlist_p, int sock);
int j_ptr_rem(jlist_p, void *j_p);
void jlist_cleanup(jlist_p);
int jlist_dump(jlist_p);
int j_dump(void *);
void j_free(void *j_p);
jinfo_p j_new(void);
int jlist_purge_sock(jlist_p jl_p, int s);
int jlist_count(void *);


#define j_rem(__l_p, __s) j_sock_rem((__l_p), (__s))

#endif
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
