/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


#ifndef  LIB_H
#define  LIB_H

#include "../config.h"

/* we want to have the 64-bit functions available but not change the existing 
 * functions; _LARGEFILE64_SOURCE accomplishes this.
 */
#define _LARGEFILE64_SOURCE

#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/socket.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <unistd.h>
#include <meta.h>
#include <req.h>
#include <desc.h>
#ifdef FS_H_IS_OK
#include <linux/fs.h> /* NR_OPEN declaration */
#endif
#include <debug.h>
#include <build_job.h>
#include <sockset.h>
#include <jlist.h>
#include <pvfs_config.h>
#include <pvfs_proto.h>


#undef PERROR
#ifdef DEBUG
#define PERROR(x) {perror(x); fflush(stderr);}
#else
#define PERROR(x);
#endif

/*PROTOTYPES*/
void *pvfs_exit(void);
int unix_lstat(const char *fn, struct stat *s_p);
int unix_stat(const char *fn, struct stat *s_p);
int unix_fstat(int fd, struct stat *s_p);
int pvfs_detect(const char *fn, char **, struct sockaddr **, int64_t *, int64_t *);

int send_mreq_saddr(struct sockaddr *saddr_p, mreq_p req_p,
	void *data_p, mack_p ack_p);
int send_mreq_dmeta(dmeta_p dir_p, mreq_p req_p, void *data_p,
	mack_p ack_p);

#ifdef SINGLE_CONNECTION
int init_iodtable(int _initial_entries, int _grow_amount);
void cleanup_iodtable(void);
int instantiate_iod_entry(struct sockaddr *saddr_p);
int add_iodtable(struct sockaddr *saddr_p);
int badiodfd(int fd);
int find_iodtable(struct sockaddr *saddr_p);
int getfd_iodtable(int slot);
int getslot_iodtable(int sockfd);
int shutdown_alliods(void);
int inc_ref_count(int slot);
int dec_ref_count(int slot);
#endif
char *dmeta2fname(dmeta_p);

int do_jobs_handle_error(jlist_p jl_p, sockset_p ss_p, int msec, int* badsock);
int do_jobs(jlist_p jl_p, sockset_p ss_p, int msec);
int do_job(int sock, jinfo_p j_p, sockset_p ss_p);

#endif
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
