/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


#ifndef BLOCKS_H
#define BLOCKS_H

/*
  blocks.h: header file for buffered multi-dimensional blocks interface
  author: W. B. Ligon III
  date: 5/27/97
  org: Clemson University
  copyright (c) 1997 Clemson University, all rights reserved.
*/

/* this stuff allows blocks code to compile for unix fs or pvfs */

#ifdef NOPVFS

/* PVFS IS NOT INCLUDED */

typedef struct file_dim file_dim, *file_dim_p;

struct file_dim {
	int dims;	/* # of dimensions */
	int esize;	/* size of element */
	int dirty;	/* flags if buffer is dirty */
	char *buffer;	/* points to block buffer */
	int *buftag;	/* points to array of ints with buffer tag */
	int *bsize;	/* size of a block */
	int *bcount;	/* number of blocks */
	int *bbuff;	/* buffering factor */
};

extern file_dim_p open_files[];
#define OPEN_FILES(fd) open_files[fd]
#define VALID(fd) (lseek(fd,0,SEEK_SET)>=0)

#define MDBI_READ read
#define MDBI_WRITE write
#define MDBI_OPEN open
#define MDBI_CLOSE close
#define MDBI_FSYNC fsync
#define MDBI_LSEEK lseek

#define PDIM 1
#define PART(f,o,g,s) do { } while (0)

#else

/* PVFS IS INCLUDED */

#include <pvfs.h>
extern fdesc_p pfds[];
#define OPEN_FILES(fd) pfds[fd]->dim_p
#define VALID(fd) pfds[fd]

#define MDBI_READ pvfs_read
#define MDBI_WRITE pvfs_write
#define MDBI_OPEN pvfs_open
#define MDBI_CLOSE pvfs_close
#define MDBI_FSYNC(fd) 0
#define MDBI_LSEEK pvfs_lseek

#define PDIM 2
#define PART(f,o,g,s)				\
	do {					\
		fpart p;			\
		p.offset = o;			\
		p.gsize = g;			\
		p.stride = s;			\
		p.gstride = 0;			\
		p.ngroups = 0;			\
		pvfs_ioctl(f, SETPART, &p);	\
		} while (0)

#endif

/* this stuff is needed for all versions of the blocks code */

#define DIM_SIZE(dim) ((dim)*4*sizeof(int))

/* values for the dirty flag */
#define DIRTY 1
#define CLEAN 0


/* mode definitions */
#define MFILE 0
#define MBUFF 1

union fpu {
        int i;
        char *p;
};

int do_write_blk(union fpu source, char *ptr, int dims, int esize,
                 int block[], int bsize[], int bcount[], int pdim, int mode);
int do_read_blk(union fpu source, char *ptr, int dims, int esize,
                int block[], int bsize[], int bcount[], int pdim, int mode);

#endif
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
