/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */



/* This file contains PVFS library call for access.   		*/
/* It will determine if file is UNIX or PVFS and then 		*/
/* make the proper request.					*/

#include <lib.h>
#include <meta.h>
#include <errno.h>

extern int pvfs_checks_disabled;

int unix_access(char *path, int mode);

int pvfs_access(char* pathname, int mode)
{
	int i=0, s;
	mack ack;
	mreq request;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;

	memset(&request, 0, sizeof(request));
	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}

	if (pvfs_checks_disabled ||
		(i = pvfs_detect(pathname, &fn, &saddr, &fs_ino, NULL)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return access(pathname, mode);
	}

	/* Prepare request for file system  */
	request.dsize = strlen(fn);
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_ACCESS;
	request.req.access.mode = mode;

	/* Send request to mgr */
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	return(0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
