/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/* This file contains PVFS library call for chown.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>
#include <errno.h>

extern fdesc_p pfds[];

int pvfs_fchown(int fd, uid_t owner, gid_t group)
{
	int s;
	struct sockaddr saddr;
	mreq request;
	mack ack;

	memset(&request, 0, sizeof(request));
	/* check for badness */
	if (fd < 0 || fd >= PVFS_NR_OPEN 
	    || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}  

	/* check for UNIX file */
	if (!pfds[fd] || pfds[fd]->fs == FS_UNIX) {
		return fchown(fd, owner, group);
	}

	/* PVFS file or directory -- Prepare request for file system  */
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_FCHOWN;
	request.dsize = 0;
	request.req.fchown.owner = (int)owner;
	request.req.fchown.group = (int)group;
	request.req.fchown.file_ino = pfds[fd]->fd.meta.u_stat.st_ino;
	request.req.fchown.fs_ino = pfds[fd]->fd.meta.fs_ino;

	/* Send request to mgr */	
	saddr = pfds[fd]->fd.meta.mgr;
	if ((s = send_mreq_saddr(&saddr, &request, NULL, &ack)) < 0) {
		PERROR("send_mreq_saddr");
		return(-1);
	}

	/* don't close manager connection */
	return (0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
