/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/* This file contains PVFS library call for fstat.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <errno.h>

extern fdesc_p pfds[];

int pvfs_fstat(int fd, struct stat *buf)
{
	int s;
	struct sockaddr saddr;
	mack ack;
	mreq req;

	memset(&req, 0, sizeof(req));
	if (fd < 0 || fd >= PVFS_NR_OPEN 
	    || (pfds[fd] && pfds[fd]->fs == FS_RESV)) 
	{
		errno = EBADF;
		return(-1);
	}  

	if (!pfds[fd] || pfds[fd]->fs == FS_UNIX) {
		return(unix_fstat(fd, buf));
	}	

	if (pfds[fd]->fs == FS_PDIR) {
		/* PVFS directory, send a stat request */
		req.uid = getuid();
		req.gid = getgid();
		req.type = MGR_STAT;
		req.dsize = strlen(pfds[fd]->fn_p);

		saddr = pfds[fd]->fd.meta.mgr;
		if ((s=send_mreq_saddr(&saddr, &req, pfds[fd]->fn_p, &ack)) < 0) {
			perror("send_mreq_saddr");
			return(-1);
		}
		COPY_STAT(buf, &ack.ack.stat.meta.u_stat);
	}
	else {
		/* otherwise it is a PVFS file -- prepare request */
		req.uid  = getuid();
		req.gid  = getgid();
		req.type = MGR_FSTAT;
		req.dsize= 0;
		req.req.fstat.meta = pfds[fd]->fd.meta;

		/* send request to mgr */
		saddr = pfds[fd]->fd.meta.mgr;
		if ((s = send_mreq_saddr(&saddr, &req, NULL, &ack)) < 0) {
			PERROR("send_mreq_saddr");
			return(-1);
		}
		COPY_STAT(buf, &ack.ack.stat.meta.u_stat);
	}

	/* don't close manager connection */

	return (0);
}

/* a pvfs library user doesn't have access to config.h to know if pvfs
 * was configured with --enable-lfs.  so pvfs_fstat64() needs to always
 * be in library, in case user defines _LARGEFILE64_SOURCE.
 */

int pvfs_fstat64(int fd, struct stat64 *buf)
{
	int s;
	struct sockaddr saddr;
	mack ack;
	mreq req;

	if (fd < 0 || fd >= PVFS_NR_OPEN 
	    || (pfds[fd] && pfds[fd]->fs == FS_RESV)) 
	{
		errno = EBADF;
		return(-1);
	}  

	if (!pfds[fd] || pfds[fd]->fs == FS_UNIX) {
#if defined (__ALPHA__) || defined (__IA64__) || !defined (LARGE_FILE_SUPPORT) || !defined (HAVE_FSTAT64)
		int r;
		struct stat sbuf;
		r = fstat(fd, &sbuf);
		COPY_STAT(buf, &sbuf);
		return r;
#else
		return(fstat64(fd, buf));
#endif
	}	

	if (pfds[fd]->fs == FS_PDIR) {
		/* PVFS directory, send a stat request */
		req.uid = getuid();
		req.gid = getgid();
		req.type = MGR_STAT;
		req.dsize = strlen(pfds[fd]->fn_p);

		saddr = pfds[fd]->fd.meta.mgr;
		if ((s=send_mreq_saddr(&saddr, &req, pfds[fd]->fn_p, &ack)) < 0) {
			perror("send_mreq_saddr");
			return(-1);
		}
		COPY_STAT(buf, &ack.ack.stat.meta.u_stat);
	}
	else {
		/* otherwise it is a PVFS file -- prepare request */
		req.uid  = getuid();
		req.gid  = getgid();
		req.type = MGR_FSTAT;
		req.dsize= 0;
		req.req.fstat.meta = pfds[fd]->fd.meta;

		/* send request to mgr */
		saddr = pfds[fd]->fd.meta.mgr;
		if ((s = send_mreq_saddr(&saddr, &req, NULL, &ack)) < 0) {
			PERROR("send_mreq_saddr");
			return(-1);
		}
		COPY_STAT(buf, &ack.ack.stat.meta.u_stat);
	}

	/* don't close manager connection */

	return (0);
}
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 *
 * vim: ts=3
 */

