/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/* This file contains PVFS library call for stat.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>
#include <errno.h>

extern int pvfs_checks_disabled;

int pvfs_ostat(char* pathname, struct stat *buf)
{
	return pvfs_stat(pathname, buf);
}

int pvfs_stat(char* pathname, struct stat *buf)
{
	int i, s;
	mack ack;
	mreq request;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino, f_ino;

	memset(&request, 0, sizeof(request));
	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}

	if (pvfs_checks_disabled) return (unix_stat(pathname, buf));

	if ((i = pvfs_detect(pathname, &fn, &saddr, &fs_ino, &f_ino)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return (unix_stat(pathname, buf));
	}

	/* Prepare request for file system  */
	request.dsize = strlen(fn);
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_STAT;

	/* Send request to mgr */
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}

	COPY_STAT(buf, &ack.ack.stat.meta.u_stat);
	return (0);
}

/* a pvfs library user doesn't have access to config.h to know if pvfs
 * was configured with --enable-lfs.  so pvfs_stat64() needs to always
 * be in library, in case user defines _LARGEFILE64_SOURCE.
 */

static inline int __unix_stat64(char* p, struct stat64 *buf)
{
#if defined (__ALPHA__) || defined (__IA64__) || !defined (LARGE_FILE_SUPPORT) || !defined (HAVE_STAT64)
        int r;
        struct stat sbuf;
        r = stat(p, &sbuf);
        COPY_STAT(buf, &sbuf);
        return r;
#else
        return(stat64(p, buf));
#endif
}

int pvfs_ostat64(char* pathname, struct stat64 *buf)
{
	return pvfs_stat64(pathname, buf);
}

int pvfs_stat64(char* pathname, struct stat64 *buf)
{
	int i, s;
	mack ack;
	mreq request;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino, f_ino;

	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}

	if (pvfs_checks_disabled) return (__unix_stat64(pathname, buf));

	if ((i = pvfs_detect(pathname, &fn, &saddr, &fs_ino, &f_ino)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return (__unix_stat64(pathname, buf));
	}

	/* Prepare request for file system  */
	request.dsize = strlen(fn);
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_STAT;

	/* Send request to mgr */
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}

	COPY_STAT(buf, &ack.ack.stat.meta.u_stat);
	return (0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
