/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


#include <lib.h>
#include <meta.h>
#ifdef HAVE_SYS_STATFS_H
#include <sys/statfs.h>
#endif
#ifdef HAVE_SYS_STATVFS_H
#include <sys/statvfs.h>
#endif
#include <pvfs_config.h>
#include <errno.h>

extern fdesc_p pfds[];
extern int pvfs_checks_disabled;

#ifdef HAVE_LINUX_STATFS
int pvfs_fstatfs(int fd, struct statfs *buf)
#else
/* fall back on statvfs() */
int pvfs_fstatfs(int fd, struct statvfs *buf)
#endif
{
	/* check for badness */
	if (fd < 0 || fd >= PVFS_NR_OPEN 
	    || (pfds[fd] && pfds[fd]->fs == FS_RESV))
	{
		errno = EBADF;
		return(-1);
	}  
	/* check for UNIX */
	if (!pfds[fd] || pfds[fd]->fs==FS_UNIX) {
#ifdef HAVE_LINUX_STATFS
		return fstatfs(fd, buf);
#else
		return fstatvfs(fd, buf);
#endif
	}

	if (pfds[fd]->fs == FS_PDIR) {
		buf->f_bsize = DEFAULT_SSIZE;
		buf->f_blocks = -1;
		buf->f_bfree = -1;
		buf->f_bavail = -1;
		buf->f_files = -1;
		buf->f_ffree = -1;
		/* don't know what to do with fsid...leaving it be... */
#ifdef HAVE_LINUX_STATFS
		buf->f_type = PVFS_SUPER_MAGIC;
		buf->f_spare[0] = -1;
		buf->f_namelen = 255; /* some random namelen */
#else
		buf->f_filler[0] = -1;
		buf->f_namemax = 255; /* some random namelen */
#endif
	}
	if (pfds[fd]->fs == FS_PVFS) {
		buf->f_bsize = pfds[fd]->fd.meta.p_stat.ssize; /* return stripe size */
		buf->f_blocks = -1;
		buf->f_bfree = -1;
		buf->f_bavail = -1;
		buf->f_files = -1;
		buf->f_ffree = -1;
		/* don't know what to do with fsid...leaving it be... */
#ifdef HAVE_LINUX_STATFS
		buf->f_type = PVFS_SUPER_MAGIC;
		buf->f_namelen = 255;
		buf->f_spare[0] = -1;
#else
		buf->f_namemax = 255;
		buf->f_filler[0] = -1;
#endif
	}

	return(0);
}

#ifdef HAVE_LINUX_STATFS
int pvfs_statfs(char *path, struct statfs *buf)
#else
/* fall back on statvfs() */
int pvfs_statfs(char *path, struct statvfs *buf)
#endif
{
	int myerr;
	mreq request;
	mack ack;
	char *fn;

	struct sockaddr *saddr;
	int64_t fs_ino;

	memset(&request, 0, sizeof(request));
	if (pvfs_checks_disabled || pvfs_detect(path, &fn, &saddr, &fs_ino, NULL) < 1)
	{
		/* not PVFS */
#ifdef HAVE_LINUX_STATFS
		return statfs(path, buf);
#else
		/* THIS REALLY ISN'T QUITE RIGHT... */
		return statvfs(path, buf);
#endif
	}

	request.dsize = strlen(fn);
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_STATFS;

	if (send_mreq_saddr(saddr, &request, fn, &ack) < 0) {
		myerr = errno;
		PERROR("pvfs_statfs: send_mreq");
		errno = myerr;
		return -1;
	}

	/* send_mreq_saddr() checks return values from mgr, so it succeeded */
	buf->f_bsize = DEFAULT_SSIZE; /* return stripe size */
	buf->f_blocks = (ack.ack.statfs.tot_bytes / (int64_t) DEFAULT_SSIZE);
	buf->f_bfree = (ack.ack.statfs.free_bytes / (int64_t) DEFAULT_SSIZE);
	buf->f_bavail = (ack.ack.statfs.free_bytes / (int64_t) DEFAULT_SSIZE);
	buf->f_files = ack.ack.statfs.tot_files;
	buf->f_ffree = ack.ack.statfs.free_files;
	/* don't know what to do with fsid...leaving it be... */
#ifdef HAVE_LINUX_STATFS
	buf->f_type = PVFS_SUPER_MAGIC;
	buf->f_namelen = ack.ack.statfs.namelen;
	buf->f_spare[0] = -1;
#else
	buf->f_namemax = ack.ack.statfs.namelen;
	buf->f_filler[0] = -1;
#endif
	

	return(0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
