/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/* This file contains PVFS library call for unlink.   	*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>

#include <string.h>
#include <meta.h>
#include <resv_name.h>
#include <errno.h>

extern int pvfs_checks_disabled;

int pvfs_unlink(const char* pathname)
{
	int i=0 /* important that this is 0 */;
	mreq request;
	mack ack;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino, f_ino;

	memset(&request, 0, sizeof(request));
	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}
	
	/* Check to see if file is a PVFS file */
	if (pvfs_checks_disabled
	|| (i = pvfs_detect(pathname, &fn, &saddr, &fs_ino, &f_ino)) != 1)
	{
		if (i == 2) /* PVFS directory */ {
			errno = EISDIR;
			return(-1);
		}
		/* check for .pvfsdir and .iodtab, reported back as UNIX files */
		if (resv_name(pathname) != 0) {
			/* pretend everything is ok if we try to remove a reserved
			 * name file, but don't really do anything.  This is here to
			 * make recursive removes work without error.
			 */
			return 0;
		}

		/* otherwise let UNIX handle giving back an error */
		return unlink(pathname);
	}

	/* Prepare request for file system  */
	request.dsize = strlen(fn);
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_UNLINK;

	/* Send request to mgr */	
	if (send_mreq_saddr(saddr, &request, fn, &ack) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	/* need to purge all tokens of this file. */

	return (0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 *
 * vim: ts=3
 */
