/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */

#include <lib.h>

#include <utime.h>
#include <sys/types.h>
#include <sys/param.h>
#include <time.h>
#include <errno.h>

extern int pvfs_checks_disabled;

/*
 * FIXME: Does this function race with reads/writes?
 */
int pvfs_utime(const char *filename, struct utimbuf *buf)
{
	mreq request;
	mack ack;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino, f_ino;

	memset(&request, 0, sizeof(request));
	/* check if this file is pvfs */
	if (pvfs_checks_disabled || pvfs_detect(filename, &fn, &saddr, &fs_ino, &f_ino) < 1)
	{
		return utime(filename, buf);
	}

	/* prepare request for manager */
   request.uid = getuid();
   request.gid = getgid();
   request.type = MGR_UTIME;
	request.dsize = strlen(fn);

	/* if buf is null, set to current time */
	if (buf) {
		request.req.utime.actime = buf->actime; /* access time */
		request.req.utime.modtime = buf->modtime; /* modification time */
	}
	else {
		request.req.utime.actime = request.req.utime.modtime
		  = time(NULL);
	}
		
   /* Send request to mgr */
	if (send_mreq_saddr(saddr, &request, fn, &ack) < 0) {
      PERROR("send_mreq");
      return(-1);
   }
	return(0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
