/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/*This file contains the close metadata file call for the PVFS*/
/*File assumes request has been allocated */

#include "../../config.h"
#include <errno.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#ifdef HAVE_LINUX_TYPES_H
#include <linux/types.h>
#endif
#include <sys/timeb.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <unistd.h>
#include <meta.h>
#include <req.h>
#include "metaio.h"

int md_close(mreq_p req, char *fname)
{
	time_t ctime;
	fmeta meta;
	int fd;

	/* get fd for meta file */
	if ((fd = meta_open(fname, O_RDWR)) < 0) {
		perror("md_close: meta_open");
		return(-1);
	}

	/* get metadata */
	if (meta_read(fd, &meta) < 0) {
		perror("md_close: meta_read");
		return(-1);
	}

	/* seek back to position 0 */
	if (lseek(fd, 0, SEEK_SET) < 0) return(-1);

	/* get current time for meta stats */
	if ((ctime = time(NULL)) < 0) {
		perror("md_close: time");
		return(-1);
	}

	/* update the modification and access times to whatever the mgr tells
	 * us
	 */
	meta.u_stat.st_atime = req->req.close.meta.u_stat.st_atime;
	meta.u_stat.st_mtime = req->req.close.meta.u_stat.st_mtime;
	/* don't change the creation time. */

/* Write metadata back */
	if (meta_write(fd, &meta) < 0) {
		perror("md_close: meta_write.");
		return(-1);
	}	

/* Close metadata file */
	if (meta_close(fd) < 0) {
		perror ("File close");
		return(-1);
	}
	
	return (0);

}	

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
