/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/*This file contains the chown metadata file call for the PVFS*/
/*Assumes that space for request has been allocated. */

#include "../../config.h"
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/param.h>
#ifdef HAVE_LINUX_TYPES_H
#include <linux/types.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <unistd.h>
#include <meta.h>
#include <req.h>

/* MD_MKDIR() - given a pathname to a directory and a dmeta structure
 * describing the directory, creates the metadata files needed in the
 * directory (currently only the .pvfsdir file).
 *
 * This function used to modify some of the structures in the dmeta_p
 * structure, but we don't do that any more.
 *
 * This function does not assume that the dirpath has a trailing /.
 *
 * put_dmeta() does ALMOST the same thing as this function, but doesn't
 * understand that the fname field should be used as part of the name
 * when we're making a new .pvfsdir file.  So this should always be used
 * when we're making a new file...
 *
 * Returns 0 on success, -1 on error.
 */
int md_mkdir(char *dirpath, dmeta_p dir)
{
	FILE *fp;
	char temp[MAXPATHLEN];

	/* create dotfile */
	strncpy(temp, dirpath, MAXPATHLEN);
	strcat(temp,"/.pvfsdir");
	umask(0033);
	if ((fp = fopen(temp, "w+")) == 0) {
		perror("Error creating dot file");
		return(-1);
	}

	/* write dotfile */
	fprintf(fp, "%Ld\n%d\n%d\n", dir->fs_ino, dir->dr_uid, dir->dr_gid);
	fprintf(fp, "%07o\n%d\n%s\n", dir->dr_mode, dir->port, dir->host);
	fprintf(fp, "%s\n%s/%s\n", dir->rd_path, dir->sd_path, dir->fname);
	fclose(fp);

	/* return success */
	return(0);

}
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
