/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/*This file contains the unlink metadata file call for the PVFS*/
/*Assumes that request and return data has been allocated */

#include "../../config.h"
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/param.h>
#ifdef HAVE_LINUX_TYPES_H
#include <linux/types.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <unistd.h>
#include <meta.h>
#include <req.h>
#include "metaio.h"

/* md_unlink(request, metadata_ptr) - takes a manager request,
 * determines the name of the metadata file to be accessed, checks
 * permissions for unlinking the file, and unlinks it.
 *
 * On failure the file is closed and -1 is returned.
 *
 * Open FD is returned on success.  This is done so that the mgr can
 * keep the inode number from being reused.  meta_close() is the appropriate
 * function to call on this FD.
 */
int md_unlink(mreq_p request, fmeta_p metar_p, char *fname)
{
	fmeta meta1;
	int fd, length;
	char temp[MAXPATHLEN];

   if ((fd = meta_open(fname, O_RDONLY)) < 0) {
		perror("md_unlink: meta_open");
		return (-1);
	}

	/* check execute permission on directory */
	/* strip off file name */
	strncpy(temp, fname, MAXPATHLEN);
	length = get_parent(temp);
	/* if length<0, CWD is being used and directory permissions will 
	 * not be checked pursuant to the UNIX method */
	if (length >= 0) {
		/* check for permissions to write to directory */
		if (meta_access(0, temp, request->uid, request->gid, X_OK) 
			< 0) {
			perror("md_unlink: meta_access");
			return(-1);
		}
	}

	/* Read metadata file */	
	if (meta_read(fd, &meta1) < 0) {
		perror("md_unlink: meta_read");
		meta_close(fd);
		return (-1);
	}

	/* Check file ownership */
	/* No need to check permissions, the user code will do that before
	 * calling this function as in UNIX */
	if ((request->uid == 0) || (request->uid == meta1.u_stat.st_uid)) {
		if (meta_unlink(fname) < 0) {
			perror ("md_unlink: meta_unlink");
			meta_close(fd);
			return(-1);
		}
	}
	else {
		fprintf(stderr, "md_unlink: permission check failed\n");
		errno = EPERM;
		meta_close(fd);
		return(-1);
	}		
			
	*metar_p = meta1;
	return(fd);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
