/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#ifdef HAVE_LINUX_TYPES_H
#include <linux/types.h>
#endif
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/param.h>
#include <meta.h>

/* put_dmeta()
 *
 * This function now assumes that it is passed a directory name.  It
 * appends "/.pvfsdir" to this name and uses that as the pvfsdir
 * filename.
 *
 * When writing out the metadata, it relies on the fname passed in
 * having the same root as the one in the dir structure.
 */
int put_dmeta(char * fname, dmeta_p dir)
{
	int length;
	char temp[MAXPATHLEN], *subdir;
	FILE *fp;
	mode_t old_umask;

   /* build name of dotfile */
   strncpy(temp, fname, MAXPATHLEN-1);
   length = strlen(temp);

   /* open dotfile */
	if (length + 9 >= MAXPATHLEN-1) {
		return(-1);
	}
   strcat(temp, "/.pvfsdir");
   old_umask = umask(0033);

	subdir = fname + strlen(dir->rd_path) + 1;

	/* should use the dmeta_xxx calls */
   if ((fp = fopen(temp, "w+")) == 0) {
      perror("Error opening dot file");
		umask(old_umask);
      return(-1);
   }

   /* write dotfile */
   fprintf(fp,"%Ld\n%d\n%d\n%07o\n%d\n%s\n%s\n%s\n\n",dir->fs_ino,
	dir->dr_uid,dir->dr_gid,dir->dr_mode,dir->port,dir->host,
	dir->rd_path,subdir);
   fclose(fp);
	umask(old_umask);

   /* return data */
   return 0;
}
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
