/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


#include <config.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

/* resv_name(name)
 *
 * Returns non-zero if a name corresponds to a PVFS reserved name, 0
 * otherwise.
 *
 * At this time, the names ".iodtab" and ".pvfsdir" are the only PVFS
 * reserved names.
 */
int resv_name(const char *name)
{
	int len, i;

	len = strlen(name); /* not including terminator */
	if (len < 7) return 0; /* our names are longer than this one */

	if (name[0] == '/' || (name[0] == '.' && name[1] == '.' && name[2] == '/')
	|| (name[0] == '.' && name[1] == '/'))
	{
		/* gotta handle directories */
		for (i = len-1; i >= 0; i--) if (name[i] == '/') break;
		/* position i of name has '/' */
		if (i > len - 7) return 0; /* again, our names are longer than this */

		i++; /* point to the first character of the name */
		if (!strcmp(&name[i], ".iodtab")
		|| !strcmp(&name[i], ".pvfsdir"))
			return 1;
	}
	else {
		/* no directories */
		if (!strcmp(name, ".iodtab") || !strcmp(name, ".pvfsdir")) return 1;
	}

	return 0;
}
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
