
/* includes for struct stat */
#include <sys/stat.h>
#include <unistd.h>

/* includes for sockaddr_in */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

/* include for the pvfs internals */
#include "meta.h"

#include <sys/types.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
/* suggested usage:  
 *  ( replace /path/to/metadata/dir with acutal metadata directory)
 *  ( run this on the mgr node )
   find /path/to/metadata/dir -type f -not -name .iodtab \
      -not -name .pvfsdir -exec fix_inodes {} \;
 */
int main(int argc, char ** argv ) {
	struct fmeta fmeta;
	struct stat stat_buf;
	int fd, err;

	if ( argc <= 1)
		exit(1);

	/* open the metadata file and read the data into the structure.  After
	 * restoring from backups, the inode number is incorrect, so figure out
	 * what the real inode should be and change that member of the
	 * pvfs_stat
	 */
	 
	if ( (fd = open(argv[1], O_RDWR, 0644) ) <= 0) {
		fprintf(stderr, "File name (%s) does not seem to be valid\n", 
				argv[1]);
		perror("Could not open file");
		exit(1);
	}
	if ( (err=read(fd, &fmeta, sizeof(fmeta))) < 0 ) {
		fprintf(stderr, "Read failed on file %s\n", argv[1]);
		perror("Could not read file");
		exit(1);
	}
	if (err != sizeof(fmeta)) {
		fprintf(stderr, "read %d, should have read %d\n", 
				err, sizeof(fmeta));
		exit(1);
	}

	stat(argv[1], &stat_buf);
	fmeta.u_stat.st_ino = stat_buf.st_ino;
	lseek(fd, 0, SEEK_SET);
	err = write(fd, &fmeta, sizeof(fmeta));
	if ( err != sizeof(fmeta)) {
		fprintf(stderr, "Short write to the file %s\n", argv[1]);
		perror("updating");
		exit(1);
	}
	close(fd);
	return(0);
}
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
