/*
	This utility converts metafile between 1.5.2 and 1.5.3
	Claude Pignol <cpignol@seismiccity.com>
*/

/* suggested usage:  
 *  ( replace /path/to/metadata/dir with acutal metadata directory)
 *  ( run this on the mgr node )
   find /path/to/metadata/dir -type f -not -name .iodtab \
      -not -name .pvfsdir -exec ./convertmeta {} \;
 */

#include <stdlib.h>
#include <string.h>
/* includes for struct stat */
#include <sys/stat.h>
#include <unistd.h>

/* includes for sockaddr_in */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

/* ********** Structures for version 1_5_2 ********** */

/* include for fmeta */

struct pvfs_filestat_1_5_2 {
    int32_t base;
    int32_t pcount;
    int32_t ssize;
    int32_t soff;
    int32_t bsize;
};

/* Structure of the metafile extracted from 1.5.2 software */
struct fmeta_1_5_2 {
    int64_t fs_ino;		/* file system root directory inode # */
    struct stat u_stat;
    struct pvfs_filestat_1_5_2 p_stat;
    int64_t fsize;
    struct sockaddr mgr;
};

/*  ********** Structures for version 1_5_3 ********** */

/* include for fmeta */

struct pvfs_stat_1_5_3 {
    uint64_t st_size;
    uint64_t st_ino;
    uint64_t st_atime;
    uint64_t st_mtime;
    uint64_t st_ctime;
    uint32_t st_mode;
    uint32_t st_uid;
    uint32_t st_gid;
    uint32_t __pad;		/* round up to 64-bit fields */
    /* unused standard stat fields: dev, nlink, rdev, blksize, blocks */
};

struct pvfs_filestat_1_5_3 {
    int32_t base;
    int32_t pcount;
    int32_t ssize;
    int32_t soff;
    int32_t bsize;
    int32_t __pad;		/* round up to 64-bit fields */
};

/* this is the structure we're actually using */
struct fmeta_1_5_3 {
    int64_t fs_ino;		/* file system root directory inode # */
    struct pvfs_stat_1_5_3 u_stat;
    struct pvfs_filestat_1_5_3 p_stat;
    int64_t fsize;
    struct sockaddr mgr;
};
#include <stdio.h>
#include <fcntl.h>
int main(int argc, char **argv)
{

    struct fmeta_1_5_2 fmeta_1_5_2;
    struct fmeta_1_5_3 fmeta_1_5_3;
    int fd, err;
    if (argc <= 1)
	exit(1);
    if ((fd = open(argv[1], O_RDONLY, 0644)) <= 0) {
	fprintf(stderr, "File name (%s) does not seem to be valid\n",
		argv[1]);
	perror("Could not open file");
	exit(1);
    }
    if ((err = read(fd, &fmeta_1_5_2, sizeof(fmeta_1_5_2))) < 0) {
	fprintf(stderr, "Read failed on file %s\n", argv[1]);
	perror("Could not read file");
	exit(1);
    }
    if (err != sizeof(fmeta_1_5_2)) {
	fprintf(stderr, "Incorrect file size read %d should be %d", err,
		sizeof(fmeta_1_5_2));
	exit(1);
    }
    close(fd);
    if ((fd = open(argv[1], O_RDWR | O_TRUNC, 0644)) <= 0) {
	fprintf(stderr, "File name (%s) does not seem to be valid\n",
		argv[1]);
	perror("Could not open file");
	exit(1);
    }
    /* build the fmeta 1.5.3 */
    fmeta_1_5_3.fs_ino = fmeta_1_5_2.fs_ino;

    fmeta_1_5_3.u_stat.st_size = fmeta_1_5_2.u_stat.st_size;
    fmeta_1_5_3.u_stat.st_ino = fmeta_1_5_2.u_stat.st_ino;
    fmeta_1_5_3.u_stat.st_atime = fmeta_1_5_2.u_stat.st_atime;
    fmeta_1_5_3.u_stat.st_mtime = fmeta_1_5_2.u_stat.st_mtime;
    fmeta_1_5_3.u_stat.st_ctime = fmeta_1_5_2.u_stat.st_ctime;
    fmeta_1_5_3.u_stat.st_mode = fmeta_1_5_2.u_stat.st_mode;
    fmeta_1_5_3.u_stat.st_uid = fmeta_1_5_2.u_stat.st_uid;
    fmeta_1_5_3.u_stat.st_gid = fmeta_1_5_2.u_stat.st_gid;
    fmeta_1_5_3.u_stat.__pad = 0;

    fmeta_1_5_3.p_stat.base = fmeta_1_5_2.p_stat.base;
    fmeta_1_5_3.p_stat.pcount = fmeta_1_5_2.p_stat.pcount;
    fmeta_1_5_3.p_stat.ssize = fmeta_1_5_2.p_stat.ssize;
    fmeta_1_5_3.p_stat.soff = fmeta_1_5_2.p_stat.soff;
    fmeta_1_5_3.p_stat.bsize = fmeta_1_5_2.p_stat.bsize;
    fmeta_1_5_3.p_stat.__pad = 0;

    /* memset this field */
    memcpy(&fmeta_1_5_3.mgr, &fmeta_1_5_2.mgr, sizeof(fmeta_1_5_2.mgr));

    err = write(fd, &fmeta_1_5_3, sizeof(fmeta_1_5_3));
    if (err != sizeof(fmeta_1_5_3)) {
	fprintf(stderr, "Error writing to the file %s\n", argv[1]);
	perror("Could not write");
	exit(1);
    }
    close(fd);
    exit(0);
}
/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 *
 * vim: ts=3
 * End:
 */ 
